/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: MIT-0
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
import * as cdk from 'aws-cdk';
import { Configuration } from 'aws-cdk/lib/settings';
import AWS = require('aws-sdk');
export declare class AssumeRoleCredentialProviderSource implements cdk.CredentialProviderSource {
    name: string;
    roleNameContextKey: string;
    config: Configuration;
    loadContextPromise: Promise<void>;
    constructor();
    /**
     * Whether the credential provider is even online
     *
     * Guaranteed to be called before any of the other functions are called.
     */
    isAvailable(): Promise<boolean>;
    private canProvideNewStyleSynthesis;
    private canProvideOldStyleSynthesis;
    /**
     * Whether the credential provider can provide credentials for the given account.
     *
     * Since we are not given the mode in this method, the most we can do here is check
     * to see whether we can obtain credentials from at least one of the roles. Because of
     * this, the method could return a false positive in some cases.
     */
    canProvideCredentials(accountId: string): Promise<boolean>;
    /**
     * Construct a credential provider for the given account and the given access mode
     *
     * Guaranteed to be called only if canProvideCredentails() returned true at some point.
     */
    getProvider(accountId: string, mode: cdk.Mode): Promise<AWS.Credentials>;
    /**
     * Look for a context key based on the mode.
     *
     * A prerequisite to using the plugin is to either create entries in
     * your cdk.context.json file or to pass in the context values from the
     * command line with the --context option.
     */
    private getRoleFromContext;
    /**
     * Get AWS region either from CDK context or from command line arguments.
     */
    private getRegion;
    /**
     * Get AWS partition based on region prefix.
     *
     * Supports three partitions:
     *  - AWS Global (`aws`),
     *  - AWS China (`aws-cn`),
     *  - AWS US Gov (`aws-us-gov`)
     */
    private getPartition;
    /**
     * Load context from cdk.context.json as well as the --context command line
     * option. Since we don't have access to the command line arguments from within the
     * cli, we get it from yargs.argv.
     *
     * We also get the verbose option on the cli so that we can match the debug
     * logs
     */
    private loadContext;
    /**
     * Creating our own CredentialProviderChain.
     *
     * The default CredentialProviderChain will check for
     * EnvironmentCredentials first, but here we're telling
     * it to first check for ECSCredentials (i.e. what CodeBuild uses).
     */
    private defaultCredentials;
    /**
     * Try to assume the specified role and return the credentials or undefined
     */
    private tryAssumeRole;
}
